#version 150 core

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;
uniform sampler2D bumpTexture;
uniform mat4 lights[8];
uniform vec3 eyePosition;

in  vec3 ex_color;
in vec2 ex_textureCoordinate;
in vec3 ex_position;
in vec3 ex_normal;
out vec4 out_color;


vec4 blinn(vec3 P, vec3 N, vec3 C, vec3 specC, float shininess)
{
	vec3 totalIntensity=vec3(0,0,0);
	int i=0;
	vec3 V=normalize(eyePosition-P);
	for (i=0;i<8;i++) //8: max number of lights
	{
		mat4 thislight=lights[i];
		if(thislight[3][0]!=0) //this light is turned on
		{
			vec3 lightposition=vec3(thislight[0][0],thislight[0][1],thislight[0][2]);
			vec3 lightcolor=vec3(thislight[1][0],thislight[1][1],thislight[1][2]);

			vec3 L = lightposition-P;
			float len=length(L);
			L=L/len;

			vec3 H=normalize(V+L);

			float kc = thislight[2][0];
			float kl = thislight[2][1];
			float kq = thislight[2][2];
			vec3 Il = lightcolor*(1.0/(kc + kl*len + kq*len*len));
		
			float diffuseFactor = dot(N, L);
			if(diffuseFactor<0)
			{
				diffuseFactor=0;
			}
			float specularFactor=diffuseFactor*pow(max(dot(N,H),0.0),shininess);
		
			vec3 diffusePart = C * diffuseFactor;
			vec3 specularPart= specC * specularFactor;
		
			totalIntensity = totalIntensity + Il* (diffusePart+specularPart);
		}
	}
        
	return vec4(totalIntensity,1);
}

void main(void)
{
	float shininess=15.0;
	vec3 specularColour=vec3(1.0,1.0,1.0);

	

	vec3 normal=ex_normal;

	out_color=blinn(ex_position,normal,texture2D(colorTexture, ex_textureCoordinate).xyz,specularColour,shininess);
}